/*
 * inflash.c
 *
 *  Created on: 2024624
 *      Author: psx
 */

#include "inflash.h"
#include "flash.h"
#include "sl_protocol.h"
#include "parameter.h"
#include <string.h>

/**
  * @brief  У㷨
  * @param
  * @retval
  */
static uint16_t configCheckFunc(uint8_t *arr_buff, uint8_t len)
{
    uint16_t crc = 0xFFFF;
    uint16_t i, j;
    for (j = 0; j < len; ++j) {
        crc = crc ^ (*arr_buff++);
        for (i = 0; i < 8; ++i) {
            if ((crc&0x0001) > 0) {
                crc = crc >> 1;
                crc = crc ^ 0xa001;
            }
            else {
                crc = crc >> 1;
            }
        }
    }
    return crc;
}

/**
 * @brief   Ϣ
 * @param   save_config_info ҪϢ
 * @retval
 */
void save_config_info(config_info *save_config_info)
{
    SPI_Flash_Write((uint8_t *)save_config_info, CONFIG_SAVE_ADDR_BEGIN, CONFIG_INFO_SIZE);
}

/**
 * @brief   Ϣ
 * @param   save_config_info ҪϢ
 * @retval
 */
void save_backups_config_info(config_info *save_config_info)
{
    SPI_Flash_Write((uint8_t *)save_config_info, CONFIG_SAVE_addr, CONFIG_INFO_SIZE);
}

/**
 * @brief   ȡϢ
 * @param   read_config_info ȡϢoutput_config_info
 */
void read_config_info(config_info *output_config_info)
{
    SPI_Flash_Read((uint8_t *)&output_config_info, CONFIG_SAVE_ADDR_BEGIN, CONFIG_INFO_SIZE);
}

/**
 * @brief   ȡݵϢ
 * @param   read_config_info ȡϢoutput_config_info
 */
static void read_backups_config_info(config_info *output_config_info)
{
    SPI_Flash_Read((uint8_t *)&output_config_info, CONFIG_SAVE_addr, CONFIG_INFO_SIZE);
}

/**
 * @brief   flashǷļļǷ𻵣˾ͽݵļûĬñ
 * @param   read_config_info1 ȡϢin_config_info
 * @retval
 *
 */
void checkFlashContent(void)
{
    config_info temp_configInfo;
    read_config_info(&temp_configInfo);

    if (temp_configInfo.crc == configCheckFunc((uint8_t *)&temp_configInfo, CONFIG_INFO_SIZE - 2)) {
        return;
    }

    read_backups_config_info(&temp_configInfo);
    if (temp_configInfo.crc == configCheckFunc((uint8_t *)&temp_configInfo, CONFIG_INFO_SIZE - 2)) {
        save_config_info(&temp_configInfo);
        return;
    }

    config_info configInfo = {
            .address = {0x11, 0x11, 0x11, 0x11, 0x11, 0x11, 0x11},
            .Access_Node_Type = 0x01,
            .Communication_Methods = 0x02,
            .gw485_Baud = 115200,
            .bat485_Baud = 115200,

            .hardwareID = {0x48, 0x59, 0x30, 0x30, 0x30, 0x31},
            .communicationID = {0x00, 0x00, 0x00, 0x01},
            .protocolType = 0x01,

            .CommunicationProtocolType = 0x00,
            .onlyPower = 0x01,

            .constantVoltageV = 14.2,
            .floatI = 0.02,
            .startSolarOpenCircuitV = 17,
            .stopSolarOpenCircuitV = 15,
            .constantVoltageChargeV = 14.4,
            .FloatChargeV = 14.2,
            .HighSideMosTemperature_stop = 100,
            .HighSideMosTemperature_end = 90,
            .HighSideMosTemperature_start = 50,
            .loopImpedance = 0.0,
            .totalElectricityConsumption = 0.0,
            .totalChargCapacity = 0.0,

            .checkSolarOpenCircuitVTime = 10,
//            .registerRefreshTime = 1,
            .sensorEnableBroadcastTime = 20,
            .outputAgainFlagTime = 10,
            .excessiveLoadFlagTime = 60,
            .eLAgainTime = 1800,
    };

    configInfo.crc = configCheckFunc((uint8_t *)&configInfo, CONFIG_INFO_SIZE - 2);

    save_config_info(&configInfo);
}

/**
 * @brief   ·迹ֵ
 * @param   loopImpedance ҪĻ·迹ֵ
 * @retval
 *
 */
void saveLoopImpedance(float_t *loopImpedance)
{
    config_info temp_configInfo;
    read_config_info(&temp_configInfo);

    temp_configInfo.loopImpedance = *loopImpedance;

    temp_configInfo.crc = configCheckFunc((uint8_t *)&temp_configInfo, CONFIG_INFO_SIZE - 2);
    save_config_info(&temp_configInfo);
}

/**
 * @brief   ܳĵֵ
 * @param   totalElectricityConsumption Ҫĵ
 *          totalChargCapacity          Ҫܳ
 * @retval
 *
 */
void saveTotalPower(float_t *totalElectricityConsumption, float_t *totalChargCapacity)
{
    config_info temp_configInfo;
    read_config_info(&temp_configInfo);

    temp_configInfo.totalElectricityConsumption = *totalElectricityConsumption;
    temp_configInfo.totalChargCapacity = *totalChargCapacity;

    temp_configInfo.crc = configCheckFunc((uint8_t *)&temp_configInfo, CONFIG_INFO_SIZE - 2);
    save_config_info(&temp_configInfo);
}

/**
 * @brief   ȡļʼ
 * @param
 * @retval
 *
 */
void config_info_start(void)
{
    checkFlashContent();

    config_info temp_configInfo;
    read_config_info(&temp_configInfo);

    g_controlParameter.constantVoltageV             = temp_configInfo.constantVoltageV;
    g_controlParameter.floatI                       = temp_configInfo.floatI;
    g_controlParameter.startSolarOpenCircuitV       = temp_configInfo.startSolarOpenCircuitV;
    g_controlParameter.stopSolarOpenCircuitV        = temp_configInfo.stopSolarOpenCircuitV;
    g_controlParameter.stopSolarOpenCircuitV        = temp_configInfo.constantVoltageChargeV;
    g_controlParameter.FloatV                       = temp_configInfo.FloatChargeV;
    g_controlParameter.loopImpedance                = temp_configInfo.loopImpedance;
    g_controlParameter.HighSideMosTemperature_stop  = temp_configInfo.HighSideMosTemperature_stop;
    g_controlParameter.HighSideMosTemperature_end   = temp_configInfo.HighSideMosTemperature_end;
    g_controlParameter.HighSideMosTemperature_start = temp_configInfo.HighSideMosTemperature_start;
    g_controlParameter.sensorEnableBroadcastTime    = temp_configInfo.sensorEnableBroadcastTime;
    g_controlParameter.checkSolarOpenCircuitVTime   = temp_configInfo.checkSolarOpenCircuitVTime;
    g_controlParameter.outputAgainFlagTime          = temp_configInfo.outputAgainFlagTime;
    g_controlParameter.excessiveLoadFlagTime        = temp_configInfo.excessiveLoadFlagTime;
    g_controlParameter.eLAgainTime                  = temp_configInfo.eLAgainTime;

    strlcpy(g_otherParameter.address, temp_configInfo.address, 7);
    g_otherParameter.Access_Node_Type               = temp_configInfo.Access_Node_Type;
    g_otherParameter.Communication_Methods          = temp_configInfo.Communication_Methods;
    strlcpy(g_otherParameter.hardwareID, temp_configInfo.hardwareID, 6);
    strlcpy(g_otherParameter.communicationID, temp_configInfo.communicationID, 4);
    g_otherParameter.CommunicationProtocolType      = temp_configInfo.CommunicationProtocolType;
    g_otherParameter.Battery_Voltage                = 0;
    g_otherParameter.Output_Voltage                 = 0;
    g_otherParameter.Charg_Current                  = 0;
    g_otherParameter.Discharg_Current               = 0;
    g_otherParameter.Input_Voltage                  = 0;
    g_otherParameter.Solar_Open_Circuit_Voltage     = 0;
    g_otherParameter.HighSideMos_Temperature        = 0;
    g_otherParameter.Solar_In_Circuit_Voltage       = 0;
    g_otherParameter.Charg_BatteryCurrent           = 0;
    g_otherParameter.totalElectricityConsumption    = temp_configInfo.totalElectricityConsumption;
    g_otherParameter.totalChargCapacity             = temp_configInfo.totalChargCapacity;
    g_otherParameter.SOC                            = 0;
    g_otherParameter.chargMos_State                 = 0;
    g_otherParameter.DischargMos_State              = 0;
    g_otherParameter.MPPT_Mode                      = 0;
    strlcpy(g_otherParameter.versionInformation, "SV01_24101501", 13);

    strlcpy(g_otherParameter.startFlagSL, "SL", 2);
    g_otherParameter.endFlagSL                      = 0x16;
    g_otherParameter.startFlagHY                    = 0x68;
    g_otherParameter.endFlagHY                      = 0x16;

    g_otherParameter.batteryState                   = 0;
    g_otherParameter.impedanceStart                 = 1;
    g_otherParameter.overTemperature                = 0;
    g_otherParameter.RegistrationRequestFlag        = 0;
    g_otherParameter.runBroadcast                   = 1;
    g_otherParameter.RegisterNumberMax              = 5;
    g_otherParameter.RegisterStartAddressMax        = 0x150;

    if (g_otherParameter.CommunicationProtocolType == 0x00) {
        g_otherParameter.gw485_Baud                 = temp_configInfo.gw485_Baud;
        g_otherParameter.bat485_Baud                = temp_configInfo.bat485_Baud;
    } else if (g_otherParameter.CommunicationProtocolType == 0x01) {
        g_otherParameter.bat485_Baud                    = temp_configInfo.bat485_Baud;
        if (temp_configInfo.protocolType == 0x01) {
            g_otherParameter.gw485_Baud             = 9600;
        } else if (temp_configInfo.protocolType == 0x02) {
            g_otherParameter.gw485_Baud             = 115200;
        }
    }
}

/**
 * @brief   жϽյļǷȷȷĻflash
 * @param
 * @retval
 *
 */
void read_and_process_config_data(void)
{
    while (strlen(config_buff) > RECV_CONFIG_INFO) {
        recv_config_info *pack = (recv_config_info *)config_buff;

        if (pack->start_Flag[0] != g_otherParameter.startFlagSL[0]
                || pack->start_Flag[1] != g_otherParameter.startFlagSL[1]) {
            memcpy(config_buff, config_buff + 1, RECV_CONFIG_INFO - 1);
            continue;
        }

//        if (pack->address[0] != g_otherParameter.address[0]
//                || pack->address[1] != g_otherParameter.address[1]
//                || pack->address[2] != g_otherParameter.address[2]
//                || pack->address[3] != g_otherParameter.address[3]
//                || pack->address[4] != g_otherParameter.address[4]
//                || pack->address[5] != g_otherParameter.address[5]
//                || pack->address[6] != g_otherParameter.address[6]) {
//            memcpy(config_buff, config_buff + 1, RECV_CONFIG_INFO - 1);
//            continue;
//        }
        uint16_t temp_u16;
        temp_u16 = (uint16_t)pack->Access_Node_Type[0] << 8
                | (uint16_t)pack->Access_Node_Type[1];
//        if (temp_u16 != POWERBOX || temp_u16 != MICROMETEOROLOGY) {
        if (temp_u16 != POWERBOX) {
            memcpy(config_buff, config_buff + 1, RECV_CONFIG_INFO - 1);
            continue;
        }

        temp_u16 = (uint16_t)pack->Communication_Methods[0] << 8
                | (uint16_t)pack->Communication_Methods[1];
//        if (temp_u16 != RS485 || temp_u16 != RJ45) {
        if (temp_u16 != RS485) {
            memcpy(config_buff, config_buff + 1, RECV_CONFIG_INFO - 1);
            continue;
        }

        uint32_t temp_u32;
        temp_u32 = (uint32_t)pack->gw485_Baud[0] << 24
                | (uint32_t)pack->gw485_Baud[1] << 16
                | (uint32_t)pack->gw485_Baud[2] << 8
                | (uint32_t)pack->gw485_Baud[3];
        if (temp_u32 != 9600 || temp_u32!= 115200) {
            memcpy(config_buff, config_buff + 1, RECV_CONFIG_INFO - 1);
            continue;
        }

        temp_u32 = (uint32_t)pack->bat485_Baud[0] << 24
                | (uint32_t)pack->bat485_Baud[1] << 16
                | (uint32_t)pack->bat485_Baud[2] << 8
                | (uint32_t)pack->bat485_Baud[3];
        if (temp_u32 != 9600 || temp_u32!= 115200 || temp_u32!= 0) {
            memcpy(config_buff, config_buff + 1, RECV_CONFIG_INFO - 1);
            continue;
        }

        if (pack->protocolType != 0x01 || pack->protocolType != 0x02) {
            memcpy(config_buff, config_buff + 1, RECV_CONFIG_INFO - 1);
            continue;
        }

        if (pack->CommunicationProtocolType != 0x00 || pack->CommunicationProtocolType != 0x01) {
            memcpy(config_buff, config_buff + 1, RECV_CONFIG_INFO - 1);
            continue;
        }

        if (pack->onlyPower != 0x00 || pack->onlyPower != 0x01) {
            memcpy(config_buff, config_buff + 1, RECV_CONFIG_INFO - 1);
            continue;
        }

        float_t temp_float;








    }
}




